#!/bin/sh

WELCOME="Welcome to the IBM Storage Host Attachment Kit installer."
CONFIRM_INSTALL_PROMPT="Would you like to proceed and install the IBM Storage Host Attachment Kit?"
ALREADY_INSTALLED="NOTE: Another version of the IBM Storage Host Attachment Kit is already installed. The installer will replace the currently installed version."
SEEKING_INSTALLED_VERSIONS="Please wait while the installer validates your existing configuration..."
CONFIRM_UPDATE_PROMPT="Would you like to replace the installed IBM Storage Host Attachment Kit?"
USER_ABORT="The installation was canceled."
ENTER_TO_PROCEED="Press [Enter] to proceed"
INSTALLING="Please wait as the IBM Storage Host Attachment Kit is being installed..."
INSTALLING_AIX="Please wait as the IBM Storage Host Attachment Kit is being installed. This may take a few minutes..."
ERROR="Installation FAILED. Please refer to install.log for more information."
SUCCESS="Installation successful."
WHATS_NEXT="Please refer to the user guide for information about how to configure this host."
INSTALLED_TOOLS="The IBM Storage Host Attachment Kit includes the following utilities:"
SEPARATOR="---------------------------------------------------------------"

LC_ALL=C
export LC_ALL

INSTALLER_DIR=`dirname $0`
cd $INSTALLER_DIR
OSNAME=`uname -s | tr '[:upper:]' '[:lower:]' | sed -e 's/-//g'`
CUR_DIR=`pwd`
INST_SCRIPT=$CUR_DIR/scripts/pkginst.$OSNAME.sh
PRE_SCRIPT=$CUR_DIR/scripts/preinst.$OSNAME.sh
POST_SCRIPT=$CUR_DIR/scripts/postinst.$OSNAME.sh
PLATFORM_COMPAT_SCRIPT=$CUR_DIR/scripts/platform.$OSNAME.sh
XPYV_UPDATE=$CUR_DIR/scripts/xpyv_upgrade.py
IS_INSTALLED_SCRIPT=$CUR_DIR/scripts/is_installed.$OSNAME.sh
MOTD_TXT=$CUR_DIR/scripts/motd.$OSNAME.txt
COMPATIBLE_PLATFORM_LIST_FILE=$CUR_DIR/packages/platform.list
LOG=$CUR_DIR/install.log

if [ "x$OSNAME" = "x" ]; then
    echo "Error obtaining platform name." 1>&2
    echo "Error obtaining platform name." >> $LOG
    exit 1
fi

echo $WELCOME 1>&2
if [ -r "$MOTD_TXT" ]; then
    echo "Displaying message in '$MOTD_TXT'" >> $LOG
    cat $MOTD_TXT
fi

while [ 1 ]; do
  echo "$CONFIRM_INSTALL_PROMPT [Y/n]:" 1>&2
  read ans
  ans=`echo $ans | tr "[:upper:]" "[:lower:]"`
  if [ "$ans" = "yes" ] || [ "$ans" = "y" ] || [ -z "$ans" ]; then
    break
  fi
  echo $SEPARATOR 1>&2
  if [ "$ans" = "no" ] || [ "$ans" = "n" ]; then
    echo $USER_ABORT 1>&2
    echo 1>&2
    exit 1
  fi
  echo "Invalid input ($ans); Valid replies are 'y' or 'yes' to confirm and 'n' or 'no' to abort."
done

# Platform compatibility check (if script exists)
if [ -x "$PLATFORM_COMPAT_SCRIPT" ] && [ -f "$COMPATIBLE_PLATFORM_LIST_FILE" ]; then
  CURRENT_PLATFORM=`$PLATFORM_COMPAT_SCRIPT`
  grep "$CURRENT_PLATFORM" $COMPATIBLE_PLATFORM_LIST_FILE > /dev/null 2>&1
  RET=$?
  if [ "$RET" != "0" ]; then
    echo "This release is not compatible with the host's platform." 1>&2
    echo "This release is compatible with the following platforms:"
    cat $COMPATIBLE_PLATFORM_LIST_FILE
    exit 1
  fi
fi

echo $SEEKING_INSTALLED_VERSIONS 1>&2
if [ -x "$IS_INSTALLED_SCRIPT" ] && $IS_INSTALLED_SCRIPT > /dev/null 2>&1; then
  echo $SEPARATOR 1>&2
  echo "$ALREADY_INSTALLED" 1>&2
  while [ 1 ]; do
    echo "$CONFIRM_UPDATE_PROMPT [Y/n]:" 1>&2
    read ans
    ans=`echo $ans | tr [:upper:] [:lower:]`
    if [ "$ans" = "yes" ] || [ "$ans" = "y" ] || [ -z "$ans" ]; then
      break
    fi
    echo $SEPARATOR 1>&2
    if [ "$ans" = "no" ] || [ "$ans" = "n" ]; then
      echo $USER_ABORT 1>&2
      echo 1>&2
      exit 1
    fi
    echo "Invalid input ($ans); Valid replies are 'y' or 'yes' to confirm and 'n' or 'no' to abort."
  done
fi

echo $SEPARATOR 1>&2
echo >> $LOG
echo "Installer: `date`: Installation started" >> $LOG

if [ "$OSNAME" = "aix" ]; then
    echo $INSTALLING_AIX 1>&2
else
    echo $INSTALLING 1>&2
fi

if [ -x "$PRE_SCRIPT" ]; then
	echo "Installer: running pre-install script '$PRE_SCRIPT'" >> $LOG
	$PRE_SCRIPT >> $LOG 2>&1 || exit $?
else
	echo "Installer: there is no pre-install script for this platform ($OSNAME)" >> $LOG
fi

echo "Installer: Installation script is '$INST_SCRIPT'" >> $LOG

for pkgname in `cat packages/pkg.list`; do
  if echo "$pkgname" | grep "xpyv-" > /dev/null; then
    echo "Installer: Checking if xpyv needs an update..." >> $LOG
    XPYV=`which xpyv 2> /dev/null`
    if ls $XPYV > /dev/null 2>&1 && $XPYV "$XPYV_UPDATE" "$pkgname" > /dev/null 2>&1; then
      echo "Installer: No need to update xpyv" >> $LOG
      continue
    else
      echo "Installer: running install script for '$pkgname'..." >> $LOG
      $INST_SCRIPT "$CUR_DIR/packages/$pkgname" >> $LOG 2>&1
      RET=$?
    fi
  else
    echo "Installer: running install script for '$pkgname'..." >> $LOG
    $INST_SCRIPT "$CUR_DIR/packages/$pkgname" >> $LOG 2>&1
    RET=$?
  fi
  # Check for error
  if [ "$RET" != "0" ]; then
    echo "Installer: Install-Script '$INST_SCRIPT' for package '$pkgname' failed." >> $LOG
    echo $ERROR 1>&2
    exit 1
  fi
done

if [ -x "$POST_SCRIPT" ]; then
	echo "Installer: running post-install script '$POST_SCRIPT'" >> $LOG
	$POST_SCRIPT >> $LOG 2>&1
    if [ "$?" != "0" ]; then
      echo "Installer: Post-Script '$POST_SCRIPT' failed." >> $LOG
      echo $ERROR 1>&2
      exit 1
    fi
else
	echo "Installer: there is no post-install script for this platform ($OSNAME)" >> $LOG
fi

echo "Installer: $SUCCESS" >> $LOG
echo $SEPARATOR 1>&2
echo $SUCCESS 1>&2
echo $WHATS_NEXT 1>&2
echo 1>&2
echo $SEPARATOR 1>&2
echo $INSTALLED_TOOLS 1>&2
cat packages/tools.list >&2
echo $SEPARATOR 1>&2
echo 1>&2
exit 0
